# -*- coding: utf-8 -*-

element_dict = {
                "Ac": "Actinium",
                "Ag": "Silver",
                "Al": "Aluminum",
                "Am": "Americium",
                "Ar": "Argon",
                "As": "Arsenic",
                "At": "Astatine",
                "Au": "Gold",
                "Ba": "Barium",
                "Be": "Beryllium",
                "Bh": "Bohrium",
                "Bi": "Bismuth",
                "Bk": "Berkelium",
                "Br": "Bromine",
                "Ca": "Calcium",
                "Cd": "Cadmium",
                "Ce": "Cerium",
                "Cf": "Californium",
                "Cl": "Chlorine",
                "Cm": "Curium",
                "Co": "Cobalt",
                "Cr": "Chromium",
                "Cs": "Cesium",
                "Cu": "Copper",
                "Db": "Dubnium",
                "Ds": "Darmstadtium",
                "Dy": "Dysprosium",
                "Er": "Erbium",
                "Es": "Einsteinium",
                "Eu": "Europium",
                "Fe": "Iron",
                "Fm": "Fermium",
                "Fr": "Francium",
                "Ga": "Gallium",
                "Gd": "Gadolinium",
                "Ge": "Germanium",
                "H": "Hydrogen",
                "He": "Helium",
                "Hf": "Hafnium",
                "Hg": "Mercury",
                "Ho": "Holmium",
                "Hs": "Hassium",
                "In": "Indium",
                "Ir": "Iridium",
                "Kr": "Krypton",
                "La": "Lanthanum",
                "Li": "Lithium",
                "Lr": "Lawrencium",
                "Lu": "Lutetium",
                "Md": "Mendelevium",
                "Mg": "Magnesium",
                "Mn": "Manganese",
                "Mo": "Molybdenum",
                "Mt": "Meitnerium",
                "Na": "Sodium",
                "Nb": "Niobium",
                "Nd": "Neodymium",
                "Ne": "Neon",
                "Ni": "Nickel",
                "No": "Nobelium",
                "Np": "Neptunium",
                "Os": "Osmium",
                "Pa": "Protactinium",
                "Pb": "Lead",
                "Pd": "Palladium",
                "Pm": "Promethium",
                "Po": "Polonium",
                "Pr": "Praseodymium",
                "Pt": "Platinum",
                "Pu": "Plutonium",
                "Ra": "Radium",
                "Rb": "Rubidium",
                "Re": "Rhenium",
                "Rf": "Rutherfordium",
                "Rg": "Roentgenium",
                "Rh": "Rhodium",
                "Rn": "Radon",
                "Ru": "Ruthenium",
                "Sb": "Antimony",
                "Sc": "Scandium",
                "Se": "Selenium",
                "Sg": "Seaborgium",
                "Si": "Silicon",
                "Sm": "Samarium",
                "Sn": "Tin",
                "Sr": "Strontium",
                "Ta": "Tantalum",
                "Tb": "Terbium",
                "Tc": "Technetium",
                "Te": "Tellurium",
                "Th": "Thorium",
                "Ti": "Titanium",
                "Tl": "Thallium",
                "Tm": "Thulium",
                "Uub": "Ununbium",
                "Uuh": "Ununhexium",
                "Uuo": "Ununoctium",
                "Uup": "Ununpentium",
                "Uuq": "Ununquadium",
                "Uus": "Ununseptium",
                "Uut": "Ununtrium",
                "Xe": "Xenon",
                "Yb": "Ytterbium",
                "Zn": "Zinc",
                "Zr": "Zirconium",
                }
