# -*- coding: utf-8 -*-

timezone_dict = {
                 "ACDT": "Australian Central Daylight Time",
                 "ACST": "Australian Central Standard Time",
                 "ACWT": "Australian Central Western Time",
                 "ADT": "Atlantic Daylight Time",
                 "ACT": "Acre Time",
                 "AEDT": "Australian Eastern Daylight Time",
                 "AEST": "Australian Eastern Standard Time",
                 "AFT": "Afghanistan Time",
                 "AKDT": "Alaska Daylight Time",
                 "AKST": "Alaska Standard Time",
                 "ALMT": "Alma-Ata Time",
                 "AMT": "Armenia Time",
                 "AMST": "Amazon Summer Time",
                 "AMST": "Armenia Summer Time",
                 "ANAT": "Anadyr Time",
                 "ANAST": "Anadyr Summer Time",
                 "AQTT": "Aqtobe Time",
                 "ART": "Argentina Time",
                 "AST": "Atlantic Standard Time",
                 "AST": "Arabia Standard Time",
                 "AWDT": "Australian Western Daylight Time",
                 "AWST": "Australian Western Standard Time",
                 "AZOT": "Azores Time",
                 "AZOST": "Azores Summer Time",
                 "AZT": "Azerbaijan Time",
                 "AZST": "Azerbaijan Summer Time",
                 "BNT": "Brunei Time",
                 "BDT": "Bangladesh Time",
                 "BOT": "Bolivia Time",
                 "BRT": "Brasilia Time",
                 "BRST": "Brasilia Summer Time",
                 "BST": "British Summer Time",
                 "BTT": "Bhutan Time",
                 "CAST": "Casey Time",
                 "CAT": "Central Africa Time",
                 "CCT": "Cocos Islands Time",
                 "CDT": "Central Daylight Time",
                 "CEDT": "Central European Daylight Time",
                 "CEST": "Central European Summer Time",
                 "CET": "Central European Time",
                 "CHADT": "Chatham Island Daylight Time",
                 "CHAST": "Chatham Island Standard Time",
                 "CHOT": "Choibalsan Time",
                 "CHOST": "Choibalsan Summer Time",
                 "CHsT": "Chamorro Standard Time",
                 "CHUT": "Chuuk Time",
                 "CIT": "Central Indonesian Time",
                 "CKT": "Cook Island Time",
                 "CLST": "Chile Summer Time",
                 "CLT": "Chile Standard Time",
                 "COT": "Columbia Time",
                 "CST": "Central Standard Time",
                 "CVT": "Cape Verde Time",
                 "CWST": "Central Western Australia Time",
                 "CXT": "Christmas Island Time",
                 "DAVT": "Davis Time",
                 "DDUT": "Dumont D 'Urville Time",
                 "DST": "Daylight Saving Time",
                 "EASST": "Eastern Island Summer Time",
                 "EAST": "Eastern Island Standard Time",
                 "EAT": "East Africa Time",
                 "ECT": "Ecuador Time",
                 "EDT": "Eastern Daylight Time",
                 "EEDT": "Eastern European Daylight Time",
                 "EEST": "Eastern European Summer Time",
                 "EET": "Eastern European Time",
                 "EGT": "Eastern Greenland Time",
                 "EGST": "Eastern Greenland Summer Time",
                 "EST": "Eastern Standard Time",
                 "EIT": "Eastern Indonesian Time",
                 "FET": "Further-Eastern European Time",
                 "FJT": "Fiji Time",
                 "FJST": "Fiji Summer Time",
                 "FKST": "Falkland Islands Summer Time",
                 "FKT": "Falkland Islands Time",
                 "FNT": "Fernando De Noronha",
                 "GALT": "Galapagos Time",
                 "GAMT": "Gambier Time",
                 "GET": "Georgia Standard Time",
                 "GFT": "French Guiana Time",
                 "GILT": "Gilbert Island Time",
                 "GMT": "Greenwich Mean Time",
                 "GST": "Gulf Standard Time",
                 "GYT": "Guyana Time",
                 "HADT": "Hawaii-Aleutian Daylight Time",
                 "HAST": "Hawaii-Aleutian Standard Time",
                 "HKT": "Hong Kong Time",
                 "HOVT": "Hovd Time",
                 "HOVST": "Hovd Summer Time",
                 "HST": "Hawaii Standard Time",
                 "ICT": "Indochina Time",
                 "IDT": "Israel Daylight Time",
                 "IOT": "Indian Chagos Time",
                 "IRDT": "Iran Daylight Time",
                 "IRKT": "Irkutsk Time",
                 "IRKST": "Irkutsk Summer Time",
                 "IRST": "Iran Standard Time",
                 "IST": "Irish Standard Time",
                 "JST": "Japan Standard Time",
                 "KGT": "Kyrgyzstan Time",
                 "KOST": "Kosrae Time",
                 "KRAT": "Krasnoyarsk Time",
                 "KRAST": "Krasnoyarsk Summer Time",
                 "KST": "Korea Standard Time",
                 "KUYT": "Kuybyshev Time",
                 "LHDT": "Lord Howe Daylight Time",
                 "LHST": "Lord Howe Standard Time",
                 "LINT": "Line Islands Time",
                 "MAGT": "Magadan Time",
                 "MAGST": "Magadan Summer Time",
                 "MART": "Marquesas Time",
                 "MAWT": "Mawson Station Time",
                 "MDT": "Mountain Daylight Time",
                 "MeST": "Metlakatla",
                 "MHT": "Marshall Islands Time",
                 "MIST": "Macquarie Island Station Time",
                 "MMT": "Myanmar Time",
                 "MSD": "Moscow Summer Time",
                 "MSK": "Moscow Standard Time",
                 "MST": "Mountain Standard Time",
                 "MUT": "Mauritius Time",
                 "MVT": "Maldives Time",
                 "MYT": "Malaysia Time",
                 "NCT": "New Caledonia Time",
                 "NDT": "Newfoundland Daylight Time",
                 "NFT": "Norfolk Time",
                 "NOVT": "Novosibirsk Time",
                 "NOVST": "Novosibirsk Summer Time",
                 "NPT": "Nepal Time",
                 "NRT": "Nauru Time",
                 "NST": "Newfoundland Standard Time",
                 "NT": "Newfoundland Time",
                 "NUT": "Niue Time",
                 "NZDT": "New Zealand Daylight Time",
                 "NZST": "New Zealand Standard Time",
                 "OMST": "Omsk Standard Time",
                 "OMSST": "Omsk Summer Time",
                 "ORAT": "Oral Time",
                 "PDT": "Pacific Daylight Time",
                 "PET": "Peru Time",
                 "PETT": "Kamchatka Time",
                 "PETST": "Kamchatka Summer Time",
                 "PGT": "Papua New Guinea Time",
                 "PHT": "Philippine Time",
                 "PHOT": "Phoenix Island Time",
                 "PKT": "Pakistan Standard Time",
                 "PMDT": "Pierre & Miquelon Daylight Time",
                 "PMST": "Pierre & Miquelon Standard Time",
                 "PONT": "Pohnpei Time",
                 "PST": "Pacific Standard Time",
                 "PWT": "Palau Time",
                 "PYT": "Paraguay Time",
                 "PYST": "Paraguay Summer Time",
                 "QYZT": "Qyzylorda Time",
                 "RET": "Reunion Time",
                 "ROTT": "Rothera",
                 "SAKT": "Sakhalin Time",
                 "SAKST": "Sakhalin Summer Time",
                 "SAMT": "Samara Time",
                 "SAST": "South Africa Standard Time",
                 "SBT": "Solomon Islands Time",
                 "SCT": "Seychelles Time",
                 "SGT": "Singapore Time",
                 "SRT": "Suriname Time",
                 "SLT": "Sri Lanka Time",
                 "SLST": "Sri Lanka Time",
                 "SRET": "Srednekolymsk Time",
                 "SST": "Samoa Standard Time",
                 "SYOT": "Syowa",
                 "TAHT": "Tahiti Time",
                 "TFT": "French Southern And Antarctic Territories Time",
                 "TJT": "Tajikistan Time",
                 "TKT": "Tokelau Time",
                 "TLT": "East Timor Time",
                 "TMT": "Turkmenistan Time",
                 "TOT": "Tonga Time",
                 "TRUT": "Truk Time",
                 "TVT": "Tuvalu Time",
                 "ULAT": "Ulaanbaatar Time",
                 "ULAST": "Ulaanbaatar Summer Time",
                 "UTC": "Coordinated Universal Time",
                 "UYST": "Uruguay Summer Time",
                 "UYT": "Uruguay Standard Time",
                 "UZT": "Uzbekistan Time",
                 "VET": "Venezuelan Standard Time",
                 "VLAT": "Vladivostok Time",
                 "VLAST": "Vladivostok Summer Time",
                 "VOLT": "Volgograd Time",
                 "VUT": "Vanuatu Time",
                 "WAKT": "Wake Island Time",
                 "WAT": "West Africa Time",
                 "WART": "West Argentina Time",
                 "WAST": "West Africa Summer Time",
                 "WDT": "Western Daylight Time",
                 "WEDT": "Western European Daylight Time",
                 "WEST": "Western European Summer Time",
                 "WET": "Western European Time",
                 "WFT": "Wallis And Futuna Time",
                 "WGT": "Western Greenland Time",
                 "WGST": "Western Greenland Summer Time",
                 "WIB": "Western Indonesian Time",
                 "WIT": "Eastern Indonesian Time",
                 "WITA": "Central Indonesian Time",
                 "WST": "Western Standard Time",
                 "WT": "Western Sahara Standard Time",
                 "YAKT": "Yakutsk Time",
                 "YAKST": "Yakutsk Summer Time",
                 "YAP": "Yap Time",
                 "YEKT": "Yekaterinburg Time",
                 "YEKST": "Yekaterinburg Summer Time",
                 }
