# -*- coding: utf-8 -*-

contractions = [
                "aren't",
                "can't",
                "could've",
                "couldn't",
                "couldn't've",
                "didn't",
                "doesn't",
                "don't",
                "gonna",
                "gotta",
                "hadn't",
                "hadn't've",
                "hasn't",
                "hasn't've",
                "haven't",
                "he'd",
                "he'd'n't",
                "he'd've",
                "he'll",
                "he’s",
                "how'd",
                "how'll",
                "how's",
                "i'd",
                "i'd'n't",
                "i'd'n't've",
                "i'd've",
                "i'll",
                "i'm",
                "i've",
                "isn't",
                "it'd",
                "it'd've",
                "it'll",
                "it's",
                "let's",
                "ma'am",
                "mightn't",
                "mightn't’ve",
                "might've",
                "mustn't",
                "mustn't've",
                "must've",
                "needn't",
                "o'clock",
                "ol'",
                "oughtn't",
                "oughtn't've",
                "shan't",
                "she'd",
                "she'd'n't",
                "she'd've",
                "she'll",
                "she's",
                "should've",
                "shouldn't",
                "shouldn't've",
                "somebody'd",
                "somebody'll",
                "somebody's",
                "someone'd",
                "someone'll",
                "someone's",
                "something'll",
                "something's",
                "'sup",
                "that'll",
                "that's",
                "that'd",
                "there'd",
                "there'd'n't",
                "there'd'n't've",
                "there'd've",
                "there're",
                "there's",
                "they'd",
                "they'dn't",
                "they'dn't've",
                "they'd've",
                "they'll",
                "they're",
                "they've",
                "'tis",
                "'twas",
                "wanna",
                "wasn't",
                "we'd",
                "we'll",
                "we're",
                "we've",
                "weren't",
                "what'd",
                "what'll",
                "what're",
                "what's",
                "what've",
                "when's",
                "where'd",
                "where's",
                "where've",
                "who'd",
                "who'd've",
                "who'll",
                "who're",
                "who's",
                "who've",
                "why'd",
                "why'll",
                "why're",
                "why's",
                "won't",
                "won't've",
                "would've",
                "wouldn't",
                "wouldn't've",
                "y'all",
                "you'd",
                "you'd've",
                "you'll",
                "you're",
                "you'ren’t",
                "you've",
                "you'ven't",
                ]
