# -*- coding: utf-8 -*-

meas_dict = {
             "%": "percent",
             "°C": "degree Celsius",
             "°F": "degree Fahrenheit",
             "F": "Fahrenheit",
             "°Re": "degree Reaumur",
             "amu": "atomic mass unit",
             "au": "astronomical unit",
             "BTC": "Bitcoin",
             "bps": "bit per second",
             "Bq": "becquerel",
             "BTU": "British Thermal Unit",
             "B/s": "byte per second",
             "bu": "bushel",
             "cbl": "cable length",
             "cal": "calorie",
             "ct": "carat",
             "cc": "cubic centimetre",
             "ccf": "centum cubic foot",
             "cg": "centigram",
             "cl": "centilitre",
             "cm": "centimetre",
             "cm²": "square centimetre",
             "cu": "cubit",
             "dm³": "cubic decimetre",
             "cm³": "cubic centimetre",
             "ft³": "cubic foot",
             "in³": "cubic inch",
             "km³": "cubic kilometre",
             "m³": "cubic metre",
             "mi³": "cubic mile",
             "mm³": "cubic millimetre",
             "pc³": "cubic parsec",
             "yd³": "cubic yard",
             "Ci": "curie",
             "dN": "decanewton",
             "dl": "decilitre",
             "dm": "decimetre",
             "deg": "degree",
             "dr": "dram",
             "eV": "electronvolt",
             "fath": "fathom",
             "ft": "foot",
             "ft²": "square foot",
             "fur": "furlong",
             "g": "gram",
             "gal": "gallon",
             "Gal": "Galileo",
             "Gb": "gigabyte",
             "GiB": "gigabyte",
             "Gbit": "gigabit",
             "Gibit": "gigabit",
             "GBq": "gigabecquerel",
             "gf": "gravet-force",
             "Gf": "grave-force",
             "gph": "gallon per hour",
             "gpm": "gallon per minute",
             "gps": "gallon per second",
             "Gy": "gray",
             "hd": "hand",
             "ha": "hectare",
             "hl": "hectolitre",
             "hPa": "hectopascal",
             "hp": "horsepower",
             "hZ": "hertz",
             "in.": "inch",
             "in²": "square inch",
             "in³": "cubic inch",
             "J": "joule",
             "K": "kelvin",
             "kHz": "kilohertz",
             "Kb": "kilobyte",
             "KiB": "kilobyte",
             "Kbit": "kilobit",
             "Kibit": "kilobit",
             "kbps": "kilobit per second",
             "kBq": "kilobecquerel",
             "kcal": "kilocalorie",
             "kg": "kilogram",
             "kJ": "kilojoule",
             "kN": "kilonewton",
             "kPa": "kilopascal",
             "kW": "kilowatt",
             "km": "kilometre",
             "km²": "square kilometre",
             "km³": "cubic kilometre",
             "kn": "knot",
             "l": "litre",
             "L": "Planck length",
             "L²": "Planck area",
             "L³": "Planck volume",
             "lb": "pound",
             "m": "metre",
             "M": "Planck mass",
             "m²": "square metre",
             "Mb": "megabyte",
             "MiB": "megabyte",
             "Mbps": "megabyte per second",
             "Mcal": "megacalorie",
             "mcg": "microgram",
             "mCi": "millicurie",
             "MBq": "megabecquerel",
             "Mbit": "megabit",
             "Mibit": "megabit",
             "MJ": "megajoule",
             "MN": "meganewton",
             "Mpa": "megapascal",
             "MW": "megawatt",
             "mkg": "metre-kilogram",
             "mg": "milligram",
             "RT": "metric refrigeration ton",
             "mi": "mile",
             "MHz": "megahertz",
             "GHz": "gigahertz",
             "mi³": "cubic mile",
             "mpg": "mile per gallon",
             "mph": "mile per hour",
             "m.p.h": "mile per hour",
             "ml": "millilitre",
             "mm": "millimetre",
             "mN": "millinewton",
             "min": "minute",
             "mm²": "square millimetre",
             "mol": "mole",
             "N": "newton",
             "nm": "nanometre",
             "Ncm": "newton centimetre",
             "Nm": "newton metre",
             "oz": "ounce",
             "ozf": "ounce force",
             "Pa": "pascal",
             "plm": "palm",
             "pc": "parsec",
             "ppm": "part per million",
             "ppb": "part per billion",
             "Pb": "petabyte",
             "pc³": "cubic parsec",
             "pdl": "poundal",
             "pt": "pint",
             "px": "pixel",
             "qt": "quart",
             "rad": "radian",
             "rd": "rod",
             "s": "second",
             "Sv": "sievert",
             "dm²": "square decimetre",
             "yd²": "square yard",
             "T": "Planck time",
             "Tbit": "terabit",
             "Tb": "terabyte",
             "tsp": "teaspoon",
             "tbsp": "tablespoon",
             "th": "thou",
             "VA": "volt-ampere",
             "V": "volt",
             "W": "watt",
             "yd": "yard",
             }

meas_dict_pl = {
                "%": "percent",
                "°C": "degrees Celsius",
                "°F": "degrees Fahrenheit",
                "F": "Fahrenheit",
                "°Re": "degrees Reaumur",
                "amu": "atomic mass units",
                "atm": "atmospheres",
                "au": "astronomical units",
                "BTC": "Bitcoins",
                "bps": "bits per second",
                "Bq": "becquerels",
                "BTU": "British Thermal Units",
                "B/s": "bytes per second",
                "bu": "bushels",
                "cbl": "cable lengths",
                "cal": "calories",
                "ct": "carats",
                "cc": "cubic centimetres",
                "ccf": "centum cubic feet",
                "cg": "centigrams",
                "cl": "centilitres",
                "cm": "centimetres",
                "cm²": "square centimetres",
                "cu": "cubits",
                "dm³": "cubic decimetres",
                "cm³": "cubic centimetres",
                "ft³": "cubic feet",
                "in³": "cubic inches",
                "km³": "cubic kilometres",
                "m³": "cubic metres",
                "mi³": "cubic miles",
                "mm³": "cubic millimetres",
                "pc³": "cubic parsecs",
                "yd³": "cubic yards",
                "Ci": "curies",
                "dN": "decanewtons",
                "dl": "decilitres",
                "dm": "decimetres",
                "deg": "degrees",
                "dr": "drams",
                "eV": "electronvolts",
                "fath": "fathoms",
                "ft": "feet",
                "ft²": "square feet",
                "fur": "furlongs",
                "g": "grams",
                "gal": "gallons",
                "Gal": "Galileos",
                "Gb": "gigabytes",
                "GiB": "gigabytes",
                "Gbit": "gigabits",
                "Gibit": "gigabits",
                "GBq": "gigabecquerels",
                "gf": "gravet-force",
                "Gf": "grave-force",
                "gph": "gallons per hour",
                "gpm": "gallons per minute",
                "gps": "gallons per second",
                "Gy": "grays",
                "hd": "hands",
                "ha": "hectares",
                "hl": "hectolitres",
                "hPa": "hectopascals",
                "hp": "horsepower",
                "hZ": "hertz",
                "in.": "inches",
                "in²": "square inches",
                "in³": "cubic inches",
                "J": "joules",
                "K": "kelvin",
                "kHz": "kilohertz",
                "Kb": "kilobytes",
                "KiB": "kilobytes",
                "Kbit": "kilobits",
                "Kibit": "kilobits",
                "kbps": "kilobits per second",
                "kBq": "kilobecquerels",
                "kcal": "kilocalories",
                "kg": "kilograms",
                "kJ": "kilojoules",
                "kN": "kilonewtons",
                "kPa": "kilopascals",
                "kW": "kilowatts",
                "km": "kilometres",
                "km²": "square kilometres",
                "km³": "cubic kilometres",
                "kn": "knots",
                "l": "litres",
                "L": "Planck length",
                "L²": "Planck area",
                "L³": "Planck volume",
                "lb": "pounds",
                "m": "metres",
                "M": "Planck mass",
                "m²": "square metres",
                "Mb": "megabytes",
                "MiB": "megabytes",
                "Mbps": "megabytes per second",
                "Mcal": "megacalories",
                "mcg": "micrograms",
                "mCi": "millicuries",
                "MBq": "megabecquerels",
                "Mbit": "megabits",
                "Mibit": "megabits",
                "MJ": "megajoules",
                "MN": "meganewtons",
                "Mpa": "megapascals",
                "MW": "megawatts",
                "mkg": "metre-kilograms",
                "mg": "milligrams",
                "RT": "metric refrigeration tonnes",
                "mi": "miles",
                "MHz": "megahertz",
                "GHz": "gigahertz",
                "mi³": "cubic miles",
                "mpg": "miles per gallon",
                "mph": "miles per hour",
                "m.p.h": "miles per hour",
                "ml": "millilitres",
                "mm": "millimetres",
                "mN": "millinewtons",
                "min": "minutes",
                "mm²": "square millimetres",
                "mol": "moles",
                "N": "newtons",
                "nm": "nanometres",
                "Ncm": "newton centimetres",
                "Nm": "newton metres",
                "oz": "ounces",
                "ozf": "ounce force",
                "Pa": "pascals",
                "plm": "palms",
                "pc": "parsecs",
                "ppm": "parts per million",
                "ppb": "parts per billion",
                "Pb": "petabytes",
                "pc³": "cubic parsecs",
                "pdl": "poundals",
                "pt": "pints",
                "px": "pixels",
                "qt": "quarts",
                "rad": "radians",
                "rd": "rods",
                "s": "seconds",
                "Sv": "sieverts",
                "dm²": "square decimetres",
                "yd²": "square yards",
                "T": "Planck time",
                "Tbit": "terabits",
                "Tb": "terabytes",
                "tsp": "teaspoons",
                "tbsp": "tablespoons",
                "th": "thou",
                "VA": "volt-amperes",
                "V": "volts",
                "W": "watts",
                "yd": "yards",
                }
