# -*- coding: utf-8 -*-

tech_words = [
              'accelerometer',
              'android',
              'archive',
              'backup',
              'barcode',
              'bitmap',
              'backlink',
              'bandwidth',
              'booting',
              'bitcoin',
              'blog',
              'bluetooth',
              'bloatware',
              'benchmark',
              'binary',
              'broadband',
              'browser',
              'byte',
              'cache',
              'cellphone',
              'cybersecurity',
              'cyberspace',
              'compression',
              'cookie',
              'deamon',
              'debug',
              'ethernet',
              'encryption',
              'email',
              'emulator',
              'ethernet',
              'facebook',
              'fiberoptic',
              'framework',
              'freeware',
              'freeview',
              'firefox',
              'firewall',
              'google',
              'gmail',
              'hardware',
              'hotmail',
              'hypertext',
              'internet',
              'java',
              'joystick',
              'keyboard',
              'keypad',
              'laptop',
              'linux',
              'macintosh',
              'malware',
              'markup',
              'megabyte',
              'microsoft',
              'modem',
              'newbie',
              'overclock',
              'phising',
              'pixel',
              'python',
              'plugin',
              'rooting',
              'server',
              'software',
              'spam',
              'supercookie',
              'telecommunication',
              'trash',
              'underclock',
              'unicode',
              'unix',
              'virus',
              'website',
              'whitespace',
              'wireless',
              'wikipedia',
              'wi-fi',
              'windows'
              ]
